mat3 isorot3(mat4 iso);
mat4 isorot4(mat4 iso);

mat3 isorotinv3(mat4 iso);
mat4 isorotinv4(mat4 iso);

vec3 isotrans3(mat4 iso);
vec4 isotrans4(mat4 iso);

vec3 isotransinv3(mat4 iso);
vec4 isotransinv4(mat4 iso);

mat4 isoinv(mat4 iso);
